﻿using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Messaging;
using System.IO;
using System.Runtime.InteropServices;


namespace MSMQPocketPC
{
	/// <summary>
	/// Summary description for form.
	/// </summary>
	public partial class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.MainMenu mainMenu1;
		protected MessageQueue deviceQ;
		public MainForm()
		{
			InitializeComponent();
		}
		private const String MSMQ_ADM = @"\windows\msmqadm.exe";
		private const String MSMQ_DRIVER_REG = @"Drivers\BuiltIn\MSMQD";
		private void btInstallMSMQ_Click(object sender, EventArgs e)
		{
			StartMSMQ();
		}


		private void StartMSMQ ()
		{
			
			CopyFilesRequired();

			
			if (!(CreateProcess(MSMQ_ADM, "status")))
			{
			
				CreateProcess(MSMQ_ADM, "register cleanup");

				
				if (CreateProcess(MSMQ_ADM, "register install"))
					MessageBox.Show("Drivers registrados correctamente");
				else
				{
					MessageBox.Show("Error = " + GetLastError().ToString());
					return;
				}

				
				if (CreateProcess(MSMQ_ADM, "register"))
					MessageBox.Show("Registro de MSMQ correcto");
				else
					MessageBox.Show("System Error = " + GetLastError().ToString());

				
				if (CreateProcess(MSMQ_ADM, "enable binary"))
					MessageBox.Show("Binarios Activados");
				else
				{
					MessageBox.Show("System Error = " + GetLastError().ToString());
					return;
				}

				
				if (CreateProcess(MSMQ_ADM, "start"))
					MessageBox.Show("Started MSMQ Successfully");
				else
				{
					IntPtr handle = ActivateDevice(MSMQ_DRIVER_REG, 0);
					CloseHandle(handle);
					if (CreateProcess(MSMQ_ADM, "status"))
						MessageBox.Show("MSMQ running");
					else
						MessageBox.Show("System Error = " + GetLastError().ToString());
				}
			}
			else
				MessageBox.Show("MSMQ ya esta activo");
		}

		private static void CopyFilesRequired()
		{
			string[] fileList = {"msmqadm.exe", "msmqadmext.dll", "msmqd.dll",
								 "msmqrt.dll", "visadm.exe" };
			string path = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase) + "\\";
			for (int i = 0; i < fileList.Length; i++)
			{
				if (!File.Exists(@"\windows\" + fileList[i]))
				{
					File.Copy(path + fileList[i], @"\windows\" + fileList[i]);
					File.Delete(path + fileList[i]);
				}
			}

		}

		public class ProcessInfo
		{
			public IntPtr hProcess;
			public IntPtr hThread;
			public Int32 ProcessId;
			public Int32 ThreadId;
		}

		[DllImport("CoreDll.DLL", SetLastError = true)]
		private extern static
			int CreateProcess(String imageName,
			String cmdLine,
			IntPtr lpProcessAttributes,
			IntPtr lpThreadAttributes,
			Int32 boolInheritHandles,
			Int32 dwCreationFlags,
			IntPtr lpEnvironment,
			IntPtr lpszCurrentDir,
			IntPtr lpsiStartInfo,
			ProcessInfo pi);

		[DllImport("CoreDll.dll")]
		private extern static
			Int32 GetLastError();

		[DllImport("CoreDll.dll")]
		private extern static
			 Int32 GetExitCodeProcess(IntPtr hProcess, out Int32 exitcode);

		[DllImport("CoreDll.dll")]
		private extern static
			 Int32 CloseHandle(IntPtr hProcess);

		[DllImport("CoreDll.dll")]
		private extern static
			IntPtr ActivateDevice(
				string lpszDevKey,
				Int32 dwClientInfo);

		[DllImport("CoreDll.dll")]
		private extern static
			Int32 WaitForSingleObject(IntPtr Handle,
			Int32 Wait);

		public static bool CreateProcess(String ExeName, String CmdLine)
		{
			Int32 INFINITE;
			unchecked { INFINITE = (int)0xFFFFFFFF; }
			ProcessInfo pi = new ProcessInfo();

			if (CreateProcess(ExeName, CmdLine, IntPtr.Zero, IntPtr.Zero,
				0, 0, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero, pi) == 0)
				return false;

			WaitForSingleObject(pi.hProcess, INFINITE);
			Int32 exitCode;

			if (GetExitCodeProcess(pi.hProcess, out exitCode) == 0)
			{
				MessageBox.Show("Failure in GetExitCodeProcess");
				CloseHandle(pi.hThread);                       
				CloseHandle(pi.hProcess);
				return false;
			}

			CloseHandle(pi.hThread);                       

			CloseHandle(pi.hProcess);

			if (exitCode != 0)
				return false;
			else
				return true;
		}

		

		private void btSend_Click(object sender, EventArgs e)
		{
			if (deviceQ == null)
			{
				string qName = @".\Private$\myQ";

				if (!MessageQueue.Exists(qName))
					MessageQueue.Create(qName);

				deviceQ = new MessageQueue(qName);
			}
			deviceQ.Send(txtSend.Text);

		}

		private void btRecv_Click(object sender, EventArgs e)
		{
			Message msg = deviceQ.Receive();
			string strMsg = (string)msg.Body;
			txtRecv.Text = strMsg;

		}

	}
}

